;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;; $Id: gnusputi.el,v 1.34 1995/04/22 11:59:03 onono Exp $
;;;;;;;;
;;;;;;;; GNUS post utility
;;;;;;;;    written by      ONO Kouichi <onono@fuka.info.waseda.ac.jp>
;;;;;;;;    Copyright (c) 1994,1995     ONO Kouichi
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;
;;;; Before using GNUS post utility (gnusputi), set the GNU Emacs
;;;; environment of your ~/.emacs as the following:
;;;;
;;;; (autoload 'gnusputi-initialize-summary-mode "gnusputi")
;;;; (autoload 'gnusputi-initialize-news-reply-mode "gnusputi")
;;;; ;; >= GNUS 3.15
;;;; (setq gnus-summary-mode-hook
;;;;      (list
;;;;       (function
;;;;        (lambda ()
;;;;          (gnusputi-initialize-summary-mode)))))
;;;; ;; If you use old GNUS (<= 3.14.4) use the following:
;;;; ;; <= GNUS 3.14.4
;;;; ;(setq gnus-Subject-mode-hook
;;;; ;     (list
;;;; ;      (function
;;;; ;       (lambda ()
;;;; ;         (gnusputi-initialize-summary-mode)))))
;;;; (setq news-reply-mode-hook
;;;;      (list
;;;;       (function
;;;;        (lambda ()
;;;;          (gnusputi-initialize-news-reply-mode)))))
;;;;
;;;; If you have already used mime.el, you should set the variable
;;;; carefully. The expression (gnusputi-initialize-news-reply-mode) is
;;;; needed to be evaluated before (mime-mode) like the following:
;;;;
;;;; (setq news-reply-mode-hook
;;;;      (list
;;;;       (function
;;;;        (lambda ()
;;;;          (gnusputi-initialize-news-reply-mode)
;;;;          (mime-mode)))))
;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;; Required libraries
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(require 'gnuspost)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;; Constants
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defconst gnusputi-version "1.3"
  "GNUS post utility version.")

(defconst gnusputi-modified-timestamp
  (let ((time-stamp "$Date: 1995/04/22 11:59:03 $")
	mod-date
	mod-date-start
	mod-date-end
	mod-time
	mod-time-start
	mod-time-end)
    (setq mod-date-start (string-match "[0-9]*/[0-9]*/[0-9]*" time-stamp))
    (if mod-date-start
	(progn
	  (setq mod-date-end (match-end 0))
	  (setq mod-time-start (string-match "[0-9]*:[0-9]*:[0-9]*" time-stamp))
	  (if mod-time-start
	      (progn
		(setq mod-time-end (match-end 0))
		(concat (substring time-stamp mod-date-start mod-date-end)
			" "
			(substring time-stamp mod-time-start mod-time-end)))
	    (substring time-stamp mod-date-start mod-date-end)))
      nil))
  "GNUS post utility modified time stamp.")

(defconst gnusputi-poster "poster"
  "The keyword poster can be written in Followup-To field
according to RFC 1036.")

(defconst gnusputi-chars-whitespace " \t\n"
  "Characters which are treated as white space characters.")

(defconst gnusputi-char-comma ?,
  "Comma character as list separator.")

(defconst gnusputi-sleep-seconds 3
  "Sleep seconds when error message is displayed.")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;; Variables
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defvar gnusputi-signature-separator "--"
  "Separator of article body and signature.")

(defvar gnusputi-truncate-signature-at-supersede nil
  "When it is non-nil, signature will be truncated if the supersede article
buffer is open from the original article. When it is nil, the original article
will be used as the supersede article.")

(defvar gnusputi-newsrc-assoc nil
  "Assoc list of read articles.
gnus-newsrc-hashtb should be kept so that both hold the same information.")

(defvar gnusputi-poster-crossable nil
  "When this variable is non-nil, the keyword poster can be crossed with
other newsgroups in Followup-To field. When nil, it must be specified
only itself. If other newsgroups already exist, gnusputi-add-followup-to
replaces them with poster.")

(defvar gnusputi-iterate-add-newsgroups t
  "When it is non-nil, gnusputi-add-newsgroups interates
to add newsgroups into Newsgroups field.
When it is nil, gnusputi-add-newsgroups add a newsgroup once.")

(defvar gnusputi-iterate-delete-newsgroups t
  "When it is non-nil, gnusputi-delete-newsgroups interates
to delete newsgroups from Newsgroups field.
When it is nil, gnusputi-delete-newsgroups deletes a newsgroup once.")

(defvar gnusputi-iterate-change-newsgroups t
  "When it is non-nil, gnusputi-change-newsgroups interates
to add newsgroups into Newsgroups field.
When it is nil, gnusputi-change-newsgroups change a newsgroup once.")

(defvar gnusputi-iterate-add-followup-to t
  "When it is non-nil, gnusputi-add-followup-to interates
to add newsgroups into Followup-To field.
When it is nil, gnusputi-add-followup-to adds a newsgroup once.")

(defvar gnusputi-iterate-delete-followup-to t
  "When it is non-nil, gnusputi-delete-followup-to interates
to delete newsgroups from Followup-To field.
When it is nil, gnusputi-delete-followup-to deletes a newsgroup once.")

(defvar gnusputi-iterate-change-followup-to t
  "When it is non-nil, gnusputi-change-followup-to interates
to add newsgroups into Followup-To field.
When it is nil, gnusputi-change-followup-to change a newsgroup once.")

(defvar gnusputi-iterate-add-distribution nil
  "When it is non-nil, gnusputi-add-distribution interates
to add distributions into Distribution field.
When it is nil, gnusputi-add-distribution adds a distribution once.")

(defvar gnusputi-iterate-delete-distribution nil
  "When it is non-nil, gnusputi-delete-distribution interates
to delete distributions from Distribution field.
When it is nil, gnusputi-delete-distribution deletes a distribution once.")

(defvar gnusputi-iterate-change-distribution nil
  "When it is non-nil, gnusputi-change-distribution interates
to add distributions into Distribution field.
When it is nil, gnusputi-change-distribution change a distribution once.")

(defvar gnusputi-default-distribution nil
  "Distribution to be inserted into Distribution field.
When it is string (not null string), gnusputi-{add,change}-distribution
use it as the default value to input distribution.
When it is non-nil (but not string), gnusputi-{add,change}-distribution
use gnus-default-distribution (<= GNUS 3.14.1) or
gnus-local-distributions (>= GNUS 3.14.4) as the default value.
When it is nil, gnusputi-{add,change}-distribution use no default values.")

(defvar gnusputi-iterate-add-reply-to nil
  "When it is non-nil, gnusputi-add-reply-to interates
to add addresses to reply into Reply-To field.
When it is nil, gnusputi-add-reply-to adds a address to reply once.")

(defvar gnusputi-iterate-delete-reply-to nil
  "When it is non-nil, gnusputi-delete-reply-to interates
to delete addresses to reply from Reply-To field.
When it is nil, gnusputi-delete-reply-to deletes a address to reply once.")

(defvar gnusputi-iterate-change-reply-to nil
  "When it is non-nil, gnusputi-change-reply-to interates
to add addresses to reply into Reply-To field.
When it is nil, gnusputi-change-reply-to adds a address to reply once.")

(defvar gnusputi-default-reply-to t
  "Address to be inserted into Reply-To field.
When it is list of string, gnusputi-{add,change}-reply-to use it
as the candidates of default values to input address to reply.
When it is string (not null string), gnusputi-{add,change}-reply-to use it
as the default value to input address to reply.
When it is non-nil (but not string), the functions use the value of
(gnus-inews-user-name) as the default to input address.
When it is nil, the functions use no default values.")

(defvar gnusputi-prev-subject-format "(was %s)"
  "The format to indicate previous subject in Subject field.")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;; Initialize Functions (mainly to set key bindings)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun gnusputi-initialize-summary-mode ()
  "Initialize summary mode environment for GNUS post utility."
  (if (boundp 'gnus-Subject-mode-map)	; <= GNUS 3.14.4
      (progn
	(or gnus-Subject-mode-map
	    (setq gnus-Subject-mode-map (make-keymap)))
	(define-key gnus-Subject-mode-map "S" 'gnusputi-summary-supersede-article))
    (if (boundp 'gnus-summary-mode-map)	; >= GNUS 3.15
	(progn
	  (or gnus-summary-mode-map
	      (setq gnus-summary-mode-map (make-keymap)))
	  (define-key gnus-summary-mode-map "S" 'gnusputi-summary-supersede-article)))))

(defun gnusputi-initialize-news-reply-mode ()
  "Initialize news reply mode environment for GNUS post utility."
  (or news-reply-mode-map
      (setq news-reply-mode-map (make-keymap)))
  (define-key news-reply-mode-map "\C-c\C-f\?"   'gnusputi-insert-newsgroup)
  (define-key news-reply-mode-map "\C-c\C-f\C-n" 'gnusputi-add-newsgroups)
  (define-key news-reply-mode-map "\C-c\C-fN"    'gnusputi-delete-newsgroups)
  (define-key news-reply-mode-map "\C-c\C-fn"    'gnusputi-change-newsgroups)
  (define-key news-reply-mode-map "\C-c\C-f\C-f" 'gnusputi-add-followup-to)
  (define-key news-reply-mode-map "\C-c\C-fF"    'gnusputi-delete-followup-to)
  (define-key news-reply-mode-map "\C-c\C-ff"    'gnusputi-change-followup-to)
  (define-key news-reply-mode-map "\C-c\C-f\C-d" 'gnusputi-add-distribution)
  (define-key news-reply-mode-map "\C-c\C-fD"    'gnusputi-delete-distribution)
  (define-key news-reply-mode-map "\C-c\C-fd"    'gnusputi-change-distribution)
  (define-key news-reply-mode-map "\C-c\C-f\C-r" 'gnusputi-add-reply-to)
  (define-key news-reply-mode-map "\C-c\C-fR"    'gnusputi-delete-reply-to)
  (define-key news-reply-mode-map "\C-c\C-fr"    'gnusputi-change-reply-to)
  (define-key news-reply-mode-map "\C-c\C-f\C-s" 'gnusputi-change-subject)
  (define-key news-reply-mode-map "\C-c\C-fs"    'gnusputi-change-subject)
  (setq gnusputi-newsrc-assoc (cons (list gnusputi-poster) gnus-newsrc-assoc)))

(fset 'gnusputi-initialize 'gnusputi-initialize-news-reply-mode)
;; for backward compatibility to version 1.1

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;; Version
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun gnusputi-version ()
  "Display GNUS post utility version."
  (interactive)
  (if (interactive-p)
      (message "%s" (gnusputi-version))
    (if gnusputi-modified-timestamp
	(format "GNUS post utility version %s (%s)" gnusputi-version gnusputi-modified-timestamp)
      (format "GNUS post utility version %s" gnusputi-version))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;; character/string/list operations
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun gnusputi-index-char (ch str)
  "Index the location of a character ch in the string str.
The origin is 0."
  (let ((len (length str))
	(idx 0)
	(pnt nil))
    (while (< idx len)
      (if (char-equal ch (elt str idx))
	  (progn
	    (setq pnt idx)
	    (setq idx len)))
      (setq idx (1+ idx)))
    pnt))

(defun gnusputi-index-string (str list)
  "Index the location of a string str in the list list.
The origin is 0."
  (let ((len (length list))
	(idx 0)
	(pnt nil))
    (while (< idx len)
      (if (string= str (elt list idx))
	  (progn
	    (setq pnt idx)
	    (setq idx len)))
      (setq idx (1+ idx)))
    pnt))

(defun gnusputi-skip-forward-chars (pnt end str)
  "Skip characters which are in the string str forward in
the current buffer."
  (while (and (< pnt end)
	      (gnusputi-index-char (char-after pnt) str))
    (setq pnt (1+ pnt)))
  pnt)

(defun gnusputi-skip-backward-chars (start pnt str)
  "Skip characters which are in the string str backward in
the current buffer."
  (while (and (> pnt start)
	      (gnusputi-index-char (char-after pnt) str))
    (setq pnt (1- pnt)))
  pnt)

(defun gnusputi-search-forward-chars (pnt end str)
  "Search a character which is in the string str forward in
the current buffer."
  (while (and (< pnt end)
	      (not (gnusputi-index-char (char-after pnt) str)))
    (setq pnt (1+ pnt)))
  (if (< pnt end)
      pnt
    nil))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;; Field/Entry operations
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun gnusputi-skip-forward-whitespace (pnt end)
  "Skip the whitespace characters forward in the current buffer."
  (gnusputi-skip-forward-chars pnt end gnusputi-chars-whitespace))

(defun gnusputi-skip-backward-whitespace (pnt end)
  "Skip the whitespace characters backward in the current buffer."
  (gnusputi-skip-backward-chars pnt end gnusputi-chars-whitespace))

(defun gnusputi-search-forward-comma (pnt end)
  "Search the comma character forward in the current buffer."
  (gnusputi-search-forward-chars pnt end (char-to-string gnusputi-char-comma)))

(defun gnusputi-header-end ()
  "Locate the end of header in the current buffer."
  (let (end
	(cur-point (point))
	(case-fold-search t))
    (goto-char (point-min))
    (search-forward (concat "\n" mail-header-separator "\n"))
    (setq end (match-beginning 0))
    (goto-char cur-point)
    end))

(defun gnusputi-field-region (field)
  "Locate the region of specified field in the current buffer."
  (let ((cur-point (point))
	(case-fold-search t))
    (goto-char (point-min))
    (if (re-search-forward (concat "^" (regexp-quote field) ":") (gnusputi-header-end) t)
	(let ((field-start (point))
	      field-end)
	  (re-search-forward "^[^ \t]" nil 'move)
	  (beginning-of-line)
	  (skip-chars-backward "\n")
	  (setq field-end (point))
	  (goto-char cur-point)
	  (list field-start field-end))
      (goto-char cur-point)
      nil)))

(defun gnusputi-list-entries-comma-separated (start end)
  "Return the list of entries separated with comma character in the region
in the current buffer."
  (let ((list-start start)
	(result-list nil)
	element)
    (while (< list-start end)
      (let ((element-start (gnusputi-skip-forward-whitespace list-start end))
	    element-end
	    comma-pnt)
	(if (null element-start) (setq element-start list-start))
	(setq comma-pnt (gnusputi-search-forward-comma element-start end))
	(if (null comma-pnt) (setq comma-pnt end))
	(setq element-end
	      (gnusputi-skip-backward-whitespace element-start (1- comma-pnt)))
	(if (null element-end) (setq element-end element-start))
	(if (not (char-equal (char-after element-end) gnusputi-char-comma))
	    (setq element-end (1+ element-end)))
	(setq element (buffer-substring element-start element-end))
	(setq list-start (1+ comma-pnt)))
      (if (not (string= element ""))
	  (setq result-list (append result-list (list element)))))
    result-list))

(defun gnusputi-get-entries-comma-separated (field)
  "Return the list of entries separated with comma character
in the specified field in the current buffer."
  (let ((field-region (gnusputi-field-region field)))
    (if field-region
	(gnusputi-list-entries-comma-separated (nth 0 field-region) (nth 1 field-region)))))

(defun gnusputi-make-alist-from-list (entries)
  "Return the association list from a list of entries."
  (let ((alist nil))
    (while entries
      (setq alist (append alist (list (list (car entries)))))
      (setq entries (cdr entries)))
    alist))

(defun gnusputi-get-alist-comma-separated (field)
  "Return the association list of entries separated with comma character
in the specified field in the current buffer."
    (gnusputi-make-alist-from-list (gnusputi-get-entries-comma-separated field)))

(defun gnusputi-make-list-entries-comma-separated (list)
  "Return the entries separated with comma character from the list."
  (if list
      (let ((str (car list))
	    (rest (cdr list)))
	(while rest
	  (setq str (concat str (char-to-string gnusputi-char-comma) (car rest)))
	  (setq rest (cdr rest)))
	str)
    ""))

(defun gnusputi-member-entry-in-list (entry list)
  "Check membership of the entry in the list."
  (let ((lst list)
	flag)
    (while lst
      (if (string= entry (car lst))
	  (progn
	    (setq flag t)
	    (setq lst nil))
	(setq flag nil)
	(setq lst (cdr lst))))
    flag))

(defun gnusputi-add-entry-into-list (entry list)
  "Add the entry into the list."
  (if (gnusputi-member-entry-in-list entry list)
      list
    (append list (list entry))))

(defun gnusputi-delete-entry-from-list (entry list)
  "Delete the entry from the list."
  (let ((lst list)
	(result nil))
    (while lst
      (let ((elem (car lst)))
	(if (not (string= entry elem))
	    (setq result (append result (list elem))))
	(setq lst (cdr lst))))
    result))

(defun gnusputi-check-entry-in-field (field entry)
  "Check the entry of the field."
  (gnusputi-index-string entry (gnusputi-get-entries-comma-separated field)))

(defun gnusputi-insert-new-field (field entry)
  "Insert the new field with entry into the current buffer."
  (goto-char (gnusputi-header-end))
  (skip-chars-backward "\n")
  (insert "\n" field ": ")
  (if entry
      (insert entry))
  t)

(defun gnusputi-add-entry-or-insert-field (field entry)
  "Add the entry into the field or insert the new field with the entry
in the current buffer."
  (let ((field-region (gnusputi-field-region field)))
    (if (null field-region)
	(gnusputi-insert-new-field field entry)
      (let ((field-start (nth 0 field-region))
	    (field-end   (nth 1 field-region))
	    entry-list)
	(setq entry-list (gnusputi-list-entries-comma-separated field-start field-end))
	(if (gnusputi-index-string entry entry-list)
	    nil
	  (goto-char field-start)
	  (delete-region field-start field-end)
	  (insert " " (gnusputi-make-list-entries-comma-separated (gnusputi-add-entry-into-list entry entry-list)))
	  t)))))

(defun gnusputi-delete-entry-from-field (field entry)
  "Delete the entry from the field in the current buffer."
  (let ((field-region (gnusputi-field-region field)))
    (if (null field-region)
	(progn
	  (ding)
	  (message (concat field " field does not exist."))
	  nil)
      (let ((field-start (nth 0 field-region))
	    (field-end   (nth 1 field-region))
	    entry-list)
	(setq entry-list (gnusputi-list-entries-comma-separated field-start field-end))
	(if (null entry-list)
	    (progn
	      (ding)
	      (message (concat "No entries in " field " field."))
	      nil)
	  (if (not (gnusputi-index-string entry entry-list))
	      (progn
		(ding)
		(message (concat entry " is not deleted because it does not exist."))
		nil)
	    (goto-char field-start)
	    (delete-region field-start field-end)
	    (insert " " (gnusputi-make-list-entries-comma-separated (gnusputi-delete-entry-from-list entry entry-list)))
	    t))))))

(defun gnusputi-change-field (field entry)
  "Change the field with entry in the current buffer."
  (let ((field-region (gnusputi-field-region field)))
    (if (null field-region)
	(gnusputi-insert-new-field field entry)
      (let ((field-start (nth 0 field-region))
	    (field-end   (nth 1 field-region)))
	(goto-char field-start)
	(if (< field-start field-end)
	    (delete-region field-start field-end))
	(insert " " entry)
	t))))

(defun gnusputi-current-entry (field)
  "Return the current entries of the field in the current buffer."
  (let ((field-region (gnusputi-field-region field)))
    (if (null field-region)
	nil
      (let ((field-start (nth 0 field-region))
	    (field-end   (nth 1 field-region)))
	(if (>= (1+ field-start) field-end)
	    nil
	  (buffer-substring (1+ field-start) field-end))))))

(defun gnusputi-equal-address (addr1 addr2)
  "T if two addresses are equal."
  (string=
   (downcase (mail-strip-quoted-names addr1))
   (downcase (mail-strip-quoted-names addr2))))

(defun gnusputi-add-addrlist (list addr)
  "Add an address to address list. If the address is already included
in the list, the address is not added."
  (if (null list)
      (list addr)
    (let ((top (car list)))
      (if (gnusputi-equal-address top addr)
	  list
	(cons top (gnusputi-add-addrlist (cdr list) addr))))))

(defun gnusputi-check-addr-in-list (addr list)
  "Check an address in the address list."
  (and list
       (or (gnusputi-equal-address addr (car list))
	   (gnusputi-check-addr-in-list addr (cdr list)))))

(defun gnusputi-select-addr (candidates addrlist)
  "Select an address from candidates list which is not in the address list."
  (if candidates
      (let ((top (car candidates)))
	(if (not (gnusputi-check-addr-in-list top addrlist))
	    top
	  (gnusputi-select-addr (cdr candidates) addrlist)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;; Interactive Functions
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Supersedes field operation
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun gnusputi-truncate-signature ()
  "Truncate signature from article."
  (interactive)
  (if (y-or-n-p (concat "Do you want to truncate signature ? "))
      (progn
	(setq gnusputi-signature-separator
	      (read-string "Signature separator: "
			   (if (and gnusputi-signature-separator
				    (stringp gnusputi-signature-separator))
			       gnusputi-signature-separator)))
	(switch-to-buffer gnus-post-news-buffer)
	(goto-char (point-max))
	(let ((signature-start)
	      (signature-end (point)))
	  ;; Skip backward a newline character of buffer-end
	  ;; to search the signature separator.
	  ;; If the separator is "", newline of buffer-end is found
	  ;; instead of the real signature separator.
	  ;; That's the reason why skips backward a newline of buffer-end.
	  (backward-char)
	  (if (not (char-equal (char-after (point)) ?\n))
	      (goto-char (point-max)))
	  (if (re-search-backward
	       (concat "^" (regexp-quote gnusputi-signature-separator) "$")
	       nil 'move) ; Search the signature separator
	      (progn
		(setq signature-start (point))
		(delete-region signature-start signature-end)
		t)
	    (goto-char (point-max))
	    (ding)
	    (message "Signature is not truncated because the separator is not found.")
	    nil)))
    (message "")
    t))

(defun gnusputi-supersede-news ()
  "Supersede an article you posted."
  (interactive)
  (if (yes-or-no-p "Do you really want to supersede this article? ")
      (let ((artbuf (current-buffer))
	    (body-start nil)
	    (body-end nil)
	    (from nil)
	    (to nil)
	    (cc nil)
	    (newsgroups nil)
	    (subject nil)
	    (message-id nil)
	    (in-reply-to nil)
	    (reply-to nil)
	    (followup-to nil)
	    (expires nil)
	    (references nil)
	    (distribution nil)
	    (organization nil)
	    (keywords nil)
	    (summary nil))
	;; Get header info. from original article.
	(save-restriction
	  (if (fboundp 'gnus-Article-show-all-headers)	 ; <= GNUS 3.14.4
	      (gnus-Article-show-all-headers)
	    (if (fboundp 'gnus-article-show-all-headers) ; >= GNUS 3.15
		(gnus-article-show-all-headers)))
	  (widen)
	  (setq body-end (point-max))
	  (goto-char (point-min))
	  (search-forward "\n\n" nil 'move)
	  (setq body-start (point))
	  (narrow-to-region (point-min) (point))
	  (setq from (mail-fetch-field "from"))
	  (setq to (mail-fetch-field "to"))
	  (setq cc (mail-fetch-field "cc"))
	  (setq newsgroups (mail-fetch-field "newsgroups"))
	  (setq subject (mail-fetch-field "subject"))
	  (setq message-id (mail-fetch-field "message-id"))
	  (setq in-reply-to (mail-fetch-field "in-reply-to"))
	  (setq reply-to (mail-fetch-field "reply-to"))
	  (setq followup-to (mail-fetch-field "followup-to"))
	  (setq expires (mail-fetch-field "expires"))
	  (setq references (mail-fetch-field "references"))
	  (setq distribution (mail-fetch-field "distribution"))
	  (setq organization (mail-fetch-field "organization"))
	  (setq keywords (mail-fetch-field "keywords"))
	  (setq summary (mail-fetch-field "summary")))
	;; Verify if the article is absolutely user's by comparing
	;; user id with value of its From field.
	(if (not
	     (string-equal
	      (downcase (mail-strip-quoted-names from))
	      (downcase (mail-strip-quoted-names (gnus-inews-user-name)))))
	    (progn
	      (ding)
	      (message "This article is not yours."))
	  ;; Make your draft
	  (pop-to-buffer gnus-post-news-buffer)
	  (news-reply-mode)
	  (gnus-overload-functions)
	  (if (and (buffer-modified-p)
		   (> (buffer-size) 0)
		   (not (y-or-n-p "Unsent article being composed; erase it? ")))
	      ;; Continue composition.
	      ;; Make news-reply-yank-original work on the current article.
	      (progn
		(setq mail-reply-buffer artbuf)
		(if (gnusputi-change-field "Supersedes" message-id)
		    (message "Supersedes field is created.")
		  (progn
		    (ding)
		    (message "Cannot create Supersedes field."))))
	    (erase-buffer)
	    (news-setup to subject in-reply-to newsgroups artbuf)
	    (gnusputi-add-entry-or-insert-field "Supersedes" message-id)
	    (if cc
		(gnusputi-add-entry-or-insert-field "Cc" cc))
	    (if reply-to
		(gnusputi-add-entry-or-insert-field "Reply-To" reply-to))
	    (if followup-to
		(gnusputi-add-entry-or-insert-field "Followup-To" followup-to))
	    (if expires
		(gnusputi-add-entry-or-insert-field "Expires" expires))
	    (if references
		(gnusputi-add-entry-or-insert-field "References" references))
	    (if distribution
		(gnusputi-add-entry-or-insert-field "Distribution" distribution))
	    (if organization
		(gnusputi-add-entry-or-insert-field "Organization" organization))
	    (if keywords
		(gnusputi-add-entry-or-insert-field "Keywords" keywords))
	    (if summary
		(gnusputi-add-entry-or-insert-field "Summary" summary))
	    ;; Handle author copy using FCC field.
	    (if gnus-author-copy
		(gnusputi-add-entry-or-insert-field "FCC" gnus-author-copy))
	    (goto-char (point-max))
	    (push-mark (point) t)	;not display the message "Mark set"
	    (insert-buffer-substring artbuf body-start body-end)
	    (if (or (not (and gnusputi-truncate-signature-at-supersede
			      (stringp gnusputi-signature-separator)))
		    (gnusputi-truncate-signature))
		(message "Mark set")))))
    (message "")))

(defun gnusputi-summary-supersede-article ()
  "Supersede an article you posted in *Summary* buffer."
  (interactive)
  (if (fboundp 'gnus-Subject-select-article)	; <= GNUS 3.14.4
      (gnus-Subject-select-article t nil)
    (if (fboundp 'gnus-summary-select-article)	; >= GNUS 3.15
	(gnus-summary-select-article t nil)))
  ;; Save window configuration.
  (setq gnus-winconf-post-news (current-window-configuration))
  (let ((art-buf
	 (or (and (boundp 'gnus-Article-buffer) gnus-Article-buffer)
	     ;; <= GNUS 3.14.4
	     (and (boundp 'gnus-article-buffer) gnus-article-buffer))))
	     ;; >= GNUS 3.15
    (if art-buf
	(unwind-protect
	    (progn
	      (switch-to-buffer art-buf)
	      (widen)
	      (delete-other-windows)
	      (gnusputi-supersede-news))
	  (or (and (eq (current-buffer) (get-buffer gnus-post-news-buffer))
		   (not (zerop (buffer-size))))
	      ;; Restore last window configuration.
	      (set-window-configuration gnus-winconf-post-news)))
      ;; We don't want to return to Article buffer later.
      (if (get-buffer art-buf)
	  (bury-buffer art-buf)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Newsgroup insert operation
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun gnusputi-insert-newsgroup ()
  "Insert a newsgroup at the point of current buffer."
  (interactive)
  (expand-abbrev)
  (let ((insert-newsgroup nil))
    (setq insert-newsgroup (completing-read "Insert Newsgroup: "
					    gnus-newsrc-assoc
					    nil
					    'require-match
					    nil ;Default newsgroup.
					    ))
    (if (not (string= insert-newsgroup ""))
	(insert insert-newsgroup))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Newsgroups field operation
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun gnusputi-add-newsgroups ()
  "Move point to end of Newsgroups field, and add newsgroups with comma.
RFC 1036 constrains the Newsgroups field to be a comma separated list of valid
newsgroups names at your site:
Newsgroups: news.misc,comp.misc,rec.misc"
  (interactive)
  (expand-abbrev)
  (let ((add-newsgroup nil)
	(added nil)
	(continue t))
    (while continue
      (setq add-newsgroup (completing-read "Add Newsgroup: "
					   gnus-newsrc-assoc
					   nil
					   'require-match
					   nil ;Default newsgroup.
					   ))
      (if (string= add-newsgroup "")
	  (setq continue nil)
	(if (not added)
	    (push-mark (point) t)) ;not display the message "Mark set"
	(if (gnusputi-add-entry-or-insert-field "Newsgroups" add-newsgroup)
	    (setq added t)
	  (if (not added) (pop-mark))
	  (ding)
	  (message (concat add-newsgroup " is not added because it already exists."))
	  (if gnusputi-iterate-add-newsgroups (sleep-for gnusputi-sleep-seconds))))
      (if (not gnusputi-iterate-add-newsgroups) (setq continue nil))
      (if added (message "Mark set")))))

(defun gnusputi-delete-newsgroups ()
  "Move point to end of Newsgroups field, and delete newsgroups.
RFC 1036 constrains the Newsgroups field to be a comma separated list of valid
newsgroups names at your site:
Newsgroups: news.misc,comp.misc,rec.misc"
  (interactive)
  (expand-abbrev)
  (let ((newsgroup-list nil)
	(delete-newsgroup nil)
	(deleted nil)
	(continue t))
    (while continue
      (setq newsgroup-list (gnusputi-get-alist-comma-separated "Newsgroups"))
      (if (not newsgroup-list)
	  (progn
	    (ding)
	    (message "No entry exist in Newsgroups field.")
	    (setq continue nil))
	(setq delete-newsgroup (completing-read "Delete Newsgroups: "
						newsgroup-list
						nil
						'require-match
						nil ;Default newsgroup.
						))
	(if (string= delete-newsgroup "")
	    (setq continue nil)
	  (if (not deleted)
	      (push-mark (point) t)) ;not display the message "Mark set"
	  (if (gnusputi-delete-entry-from-field "Newsgroups" delete-newsgroup)
	      (setq deleted t)
	    (if (not deleted) (pop-mark))
	    (if gnusputi-iterate-delete-newsgroups (sleep-for gnusputi-sleep-seconds))))
	(if (not gnusputi-iterate-delete-newsgroups) (setq continue nil))
	(if deleted (message "Mark set"))))))

(defun gnusputi-change-newsgroups ()
  "Move point to end of Newsgroups field, and change newsgroup entry."
  (interactive)
  (expand-abbrev)
  (let ((change-newsgroup nil)
	(changed nil)
	(continue t))
    (while continue
      (setq change-newsgroup (completing-read (if changed
						  "Add Newsgroups: "
						"Change Newsgroups: ")
					      gnus-newsrc-assoc
					      nil
					      'require-match
					      nil ;Default newsgroup.
					      ))
      (if (string= change-newsgroup "")
	  (setq continue nil)
	(if (not changed)
	    (progn
	      (push-mark (point) t) ;not display the message "Mark set"
	      (if (gnusputi-change-field "Newsgroups" change-newsgroup)
		  (setq changed t)
		(if (not changed) (pop-mark))
		(ding)
		(message (concat "Cannot change Newsgroups field as \"" change-newsgroup "\"."))
		(if gnusputi-iterate-change-newsgroups (sleep-for gnusputi-sleep-seconds))))
	  (if (not (gnusputi-add-entry-or-insert-field "Newsgroups" change-newsgroup))
	      (progn
		(ding)
		(message (concat change-newsgroup " is not added because it already exists."))
		(if gnusputi-iterate-change-newsgroups (sleep-for gnusputi-sleep-seconds))))))
      (if (not gnusputi-iterate-change-newsgroups) (setq continue nil))
      (if changed (message "Mark set")))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Followup-To field operation
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun gnusputi-add-followup-to ()
  "Move point to end of Followup-To field, and add newsgroups with comma.
Create the field with a newsgroup if none.
One usually requests followups to only one newsgroup.
RFC 1036 constrains the Followup-To field to be a comma separated list of
valid newsgroups names at your site, that are also in the Newsgroups field:
Newsgroups: news.misc,comp.misc,rec.misc,misc.misc,soc.misc
Followup-To: news.misc,comp.misc,rec.misc"
  (interactive)
  (expand-abbrev)
  (if (and (not gnusputi-poster-crossable)
	   (gnusputi-check-entry-in-field "Followup-To" gnusputi-poster))
      (progn
	(ding)
	(message (concat "Cannot add newsgroup because \"" gnusputi-poster "\" is already specified.")))
    (let ((add-newsgroup nil)
	  (added nil)
	  (continue t))
      (while continue
	(setq add-newsgroup (completing-read "Add Followup-To: "
					     gnusputi-newsrc-assoc
					     nil
					     'require-match
					     nil ;Default newsgroup.
					     ))
	(if (string= add-newsgroup "")
	    (setq continue nil)
	  (if (not added)
	      (push-mark (point) t)) ;not display the message "Mark set"
	  (if (and (not gnusputi-poster-crossable)
		   (string= add-newsgroup gnusputi-poster))
	      (if (gnusputi-change-field "Followup-To" add-newsgroup)
		  (progn
		    (message (concat "change Followup-To field with \"" gnusputi-poster "\""))
		    (setq continue nil))
		(if (not added) (pop-mark))
		(ding)
		(message (concat "Cannot change Followup-To field as \"" add-newsgroup "\"."))
		(if gnusputi-iterate-add-followup-to (sleep-for gnusputi-sleep-seconds)))
	    (if (gnusputi-add-entry-or-insert-field "Followup-To" add-newsgroup)
		(setq added t)
	      (if (not added) (pop-mark))
	      (ding)
	      (message (concat add-newsgroup " is not added because it already exists."))
	      (if gnusputi-iterate-add-followup-to (sleep-for gnusputi-sleep-seconds)))
	    (if (not gnusputi-iterate-add-followup-to) (setq continue nil))
	    (if added (message "Mark set"))))))))

(defun gnusputi-delete-followup-to ()
  "Move point to end of Followup-To field, and delete newsgroups.
RFC 1036 constrains the Followup-To field to be a comma separated list of valid
newsgroups names at your site:
Followup-To: news.misc,comp.misc,rec.misc"
  (interactive)
  (expand-abbrev)
  (let ((newsgroup-list nil)
	(delete-newsgroup nil)
	(deleted nil)
	(continue t))
    (while continue
      (setq newsgroup-list (gnusputi-get-alist-comma-separated "Followup-To"))
      (if (not newsgroup-list)
	  (progn
	    (ding)
	    (message "No entry exist in Followup-To field.")
	    (setq continue nil))
	(setq delete-newsgroup (completing-read "Delete Followup-To: "
						newsgroup-list
						nil
						'require-match
						nil ;Default newsgroup.
						))
	(if (string= delete-newsgroup "")
	    (setq continue nil)
	  (if (not deleted)
	      (push-mark (point) t)) ;not display the message "Mark set"
	  (if (gnusputi-delete-entry-from-field "Followup-To" delete-newsgroup)
	      (setq deleted t)
	    (if (not deleted) (pop-mark))
	    (if gnusputi-iterate-delete-followup-to (sleep-for gnusputi-sleep-seconds))))
	(if (not gnusputi-iterate-delete-followup-to) (setq continue nil))
	(if deleted (message "Mark set"))))))

(defun gnusputi-change-followup-to ()
  "Move point to end of Followup-To field, and change newsgroup entry."
  (interactive)
  (expand-abbrev)
  (let ((change-newsgroup nil)
	(changed nil)
	(continue t))
    (while continue
      (setq change-newsgroup (completing-read (if changed
						  "Add Followup-To: "
						"Change Followup-To: ")
					      gnusputi-newsrc-assoc
					      nil
					      'require-match
					      nil ;Default newsgroup.
					      ))
      (if (string= change-newsgroup "")
	  (setq continue nil)
	(if (not changed)
	    (progn
	      (push-mark (point) t) ;not display the message "Mark set"
	      (if (gnusputi-change-field "Followup-To" change-newsgroup)
		  (setq changed t)
		(progn
		  (pop-mark)
		  (ding)
		  (message (concat "Cannot change Followup-To field as \"" change-newsgroup "\"."))
		  (if gnusputi-iterate-change-followup-to (sleep-for gnusputi-sleep-seconds))))
	      (if (and (not gnusputi-poster-crossable)
		       (string= change-newsgroup gnusputi-poster)
		       changed)
		  (progn
		    (message (concat "change Followup-To field with \"" gnusputi-poster "\""))
		    (setq continue nil))))
	  (if (and (not gnusputi-poster-crossable)
		   (string= change-newsgroup gnusputi-poster))
	      (if (gnusputi-change-field "Followup-To" change-newsgroup)
		  (progn
		    (message (concat "change Followup-To field with \"" gnusputi-poster "\""))
		    (setq continue nil))
		(ding)
		(message (concat "Cannot change Followup-To field as \"" change-newsgroup "\"."))
		(if gnusputi-iterate-change-followup-to (sleep-for gnusputi-sleep-seconds)))
	    (if (not (gnusputi-add-entry-or-insert-field "Followup-To" change-newsgroup))
		(progn
		  (ding)
		  (message (concat change-newsgroup " is not added because it already exists."))
		  (if gnusputi-iterate-change-followup-to (sleep-for gnusputi-sleep-seconds)))))))
      (if (not gnusputi-iterate-change-followup-to) (setq continue nil))
      (if changed (message "Mark set")))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Distribution field operation
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun gnusputi-add-distribution ()
  "Move point to end of option Distribution field,
and add distributions with comma.
Create the field with a distribution if none.
Without this field the posting goes to all of
USENET.  The field is used to restrict the posting to parts of USENET."
  (interactive)
  (expand-abbrev)
  (let ((add-distribution nil)
	(added nil)
	(continue t))
    (while continue
      (setq add-distribution
	    (if (and (boundp 'gnus-distribution-list)
		     (listp gnus-distribution-list))
		(completing-read "Add Distribution: "
				 gnus-distribution-list
				 nil
				 nil
				 (if gnusputi-default-distribution
				     (if (stringp gnusputi-default-distribution)
					 (if (not (string= gnusputi-default-distribution ""))
					     gnusputi-default-distribution)
				       (if (and (boundp 'gnus-local-distributions)
						(listp gnus-local-distributions))
					   (car gnus-local-distributions)
					 (if (boundp 'gnus-default-distribution)
					     gnus-default-distribution))))
					;Default distribution.
				 )
	      (read-string "Add Distribution: ")))
      (if (string= add-distribution "")
	  (setq continue nil)
	(if (not added)
	    (push-mark (point) t)) ;not display the message "Mark set"
	(if (gnusputi-add-entry-or-insert-field "Distribution" add-distribution)
	    (setq added t)
	  (if (not added) (pop-mark))
	  (ding)
	  (message (concat add-distribution " is not added because it already exists."))
	  (if gnusputi-iterate-add-distribution (sleep-for gnusputi-sleep-seconds))))
      (if (not gnusputi-iterate-add-distribution) (setq continue nil))
      (if added (message "Mark set")))))

(defun gnusputi-delete-distribution ()
  "Move point to end of optional Distribution field, and delete distributions.
Without this field the posting goes to all of USENET.
The field is used to restrict the posting to parts of USENET."
  (interactive)
  (expand-abbrev)
  (let ((distribution-list nil)
	(delete-distribution nil)
	(deleted nil)
	(continue t))
    (while continue
      (setq distribution-list (gnusputi-get-alist-comma-separated "Distribution"))
      (if (not distribution-list)
	  (progn
	    (ding)
	    (message "No entry exist in Distribution field.")
	    (setq continue nil))
	(setq delete-distribution (completing-read "Delete Distribution: "
						   distribution-list
						   nil
						   'require-match
						   nil ;Default distribution.
						   ))
	(if (string= delete-distribution "")
	    (setq continue nil)
	  (if (not deleted)
	      (push-mark (point) t)) ;not display the message "Mark set"
	  (if (gnusputi-delete-entry-from-field "Distribution" delete-distribution)
	      (setq deleted t)
	    (if (not deleted) (pop-mark))
	    (if gnusputi-iterate-delete-distribution (sleep-for gnusputi-sleep-seconds))))
	(if (not gnusputi-iterate-delete-distribution) (setq continue nil))
	(if deleted (message "Mark set"))))))

(defun gnusputi-change-distribution ()
  "Move point to end of Distribution field, and change distribution entry."
  (interactive)
  (expand-abbrev)
  (let ((change-distribution nil)
	(changed nil)
	(continue t))
    (while continue
      (setq change-distribution
	    (if (and (boundp 'gnus-distribution-list)
		     (listp gnus-distribution-list))
		(completing-read (if changed
				     "Add Distribution: "
				   "Change Distribution: ")
				 gnus-distribution-list
				 nil
				 nil
				 (if gnusputi-default-distribution
				     (if (stringp gnusputi-default-distribution)
					 (if (not (string= gnusputi-default-distribution ""))
					     gnusputi-default-distribution)
				       (if (and (boundp 'gnus-local-distributions)
						(listp gnus-local-distributions))
					   (car gnus-local-distributions)
					 (if (boundp 'gnus-default-distribution)
					     gnus-default-distribution))))
					;Default distribution.
				 )
	      (read-string (if changed
			       "Add Distribution: "
			     "Change Distribution: "))))
      (if (string= change-distribution "")
	  (setq continue nil)
	(if (not changed)
	    (progn
	      (push-mark (point) t) ;not display the message "Mark set"
	      (if (gnusputi-change-field "Distribution" change-distribution)
		  (setq changed t)
		(if (not changed) (pop-mark))
		(ding)
		(message (concat "Cannot change Distribution field as \"" change-distribution "\"."))
		(if gnusputi-iterate-change-distribution (sleep-for gnusputi-sleep-seconds))))
	  (if (not (gnusputi-add-entry-or-insert-field "Distribution" change-distribution))
	      (progn
		(ding)
		(message (concat change-distribution " is not added because it already exists."))
		(if gnusputi-iterate-change-distribution (sleep-for gnusputi-sleep-seconds))))))
      (if (not gnusputi-iterate-change-distribution) (setq continue nil))
      (if changed (message "Mark set")))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Reply-To field operation
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun gnusputi-add-reply-to ()
  "Move point to end of Reply-To field, and add addresses to reply with comma.
The Reply-To field to be a comma separated list of valid addresses
under RFC 1036/822.
Reply-To: foo@xxx.edu (Foo Foo),bar@xxx.edu (Bar Bar)"
  (interactive)
  (expand-abbrev)
  (let ((add-reply-to nil)
	(default-reply-to (gnus-inews-user-name))
	(addr-list nil)
	(reply-to-candidates nil)
	(reply-to-list nil)
	(added nil)
	(continue t))
    (if gnusputi-default-reply-to
	(setq addr-list
	      (if (stringp gnusputi-default-reply-to)
		  (list gnusputi-default-reply-to)
		(if (listp gnusputi-default-reply-to)
		    gnusputi-default-reply-to
		  (list default-reply-to)))))
    (setq reply-to-candidates (gnusputi-add-addrlist addr-list default-reply-to))
    (while continue
      (setq reply-to-list (gnusputi-get-entries-comma-separated "Reply-To"))
      (setq add-reply-to
	    (completing-read "Add Reply-To: "
			     (gnusputi-make-alist-from-list reply-to-candidates)
			     nil
			     nil
			     (gnusputi-select-addr reply-to-candidates reply-to-list)
			     ))
      (if (string= add-reply-to "")
	  (setq continue nil)
	(if (gnusputi-check-addr-in-list add-reply-to reply-to-list)
	    (progn
	      (ding)
	      (message (concat add-reply-to " is not added because it already exists."))
	      (if gnusputi-iterate-add-reply-to (sleep-for gnusputi-sleep-seconds)))
	  (if (not added)
	      (push-mark (point) t)) ;not display the message "Mark set"
	  (if (gnusputi-add-entry-or-insert-field "Reply-To" add-reply-to)
	      (setq added t)
	    (if (not added) (pop-mark))
	    (ding)
	    (message (concat "Cannot add an address '" add-reply-to "' into Reply-To field."))
	    (if gnusputi-iterate-add-reply-to (sleep-for gnusputi-sleep-seconds)))))
      (if (not gnusputi-iterate-add-reply-to) (setq continue nil))
      (if added (message "Mark set")))))

(defun gnusputi-delete-reply-to ()
  "Move point to end of Reply-To field, and delete addresses to reply with comma.
The Reply-To field to be a comma separated list of valid addresses
under RFC 1036/822.
Reply-To: foo@xxx.edu (Foo Foo),bar@xxx.edu (Bar Bar)"
  (interactive)
  (expand-abbrev)
  (let ((reply-to-list nil)
	(delete-reply-to nil)
	(deleted nil)
	(continue t))
    (while continue
      (setq reply-to-list (gnusputi-get-alist-comma-separated "Reply-To"))
      (if (not reply-to-list)
	  (progn
	    (ding)
	    (message "No entry exist in Reply-To field.")
	    (setq continue nil))
	(setq delete-reply-to
	      (completing-read "Delete Reply-To: "
			       reply-to-list
			       nil
			       'require-match
			       nil ;Default address
			       ))
	(if (string= delete-reply-to "")
	    (setq continue nil)
	  (if (not deleted)
	      (push-mark (point) t)) ;not display the message "Mark set"
	  (if (gnusputi-delete-entry-from-field "Reply-To" delete-reply-to)
	      (setq deleted t)
	    (if (not deleted) (pop-mark))
	    (if gnusputi-iterate-delete-reply-to (sleep-for gnusputi-sleep-seconds))))
	(if (not gnusputi-iterate-delete-reply-to) (setq continue nil))
	(if deleted (message "Mark set"))))))

(defun gnusputi-change-reply-to ()
  "Move point to end of Reply-To field, and change addresses to reply with comma.
The Reply-To field to be a comma separated list of valid addresses
under RFC 1036/822.
Reply-To: foo@xxx.edu (Foo Foo),bar@xxx.edu (Bar Bar)"
  (interactive)
  (expand-abbrev)
  (let ((change-reply-to nil)
	(default-reply-to (gnus-inews-user-name))
	(addr-list nil)
	(reply-to-candidates nil)
	(reply-to-list nil)
	(changed nil)
	(continue t))
    (if gnusputi-default-reply-to
	(setq addr-list
	      (if (stringp gnusputi-default-reply-to)
		  (list gnusputi-default-reply-to)
		(if (listp gnusputi-default-reply-to)
		    gnusputi-default-reply-to
		  (list default-reply-to)))))
    (setq reply-to-candidates (gnusputi-add-addrlist addr-list default-reply-to))
    (while continue
      (setq reply-to-list (gnusputi-get-entries-comma-separated "Reply-To"))
      (setq change-reply-to
	    (completing-read (if changed
				 "Add Reply-To: "
			       "Change Reply-To: ")
			     (gnusputi-make-alist-from-list reply-to-candidates)
			     nil
			     nil
			     (if changed
				 (gnusputi-select-addr reply-to-candidates reply-to-list)
			       (car reply-to-candidates))
			     ))
      (if (string= change-reply-to "")
	  (setq continue nil)
	(if (not changed)
	    (progn
	      (push-mark (point) t) ;not display the message "Mark set"
	      (if (gnusputi-change-field "Reply-To" change-reply-to)
		  (setq changed t)
		(if (not changed) (pop-mark))
		(ding)
		(message (concat "Cannot change Reply-To field as \"" change-reply-to "\"."))
		(if gnusputi-iterate-change-reply-to (sleep-for gnusputi-sleep-seconds))))
	  (if (gnusputi-check-addr-in-list change-reply-to reply-to-list)
	      (progn
		(ding)
		(message (concat change-reply-to " is not added because it already exists."))
		(if gnusputi-iterate-change-reply-to (sleep-for gnusputi-sleep-seconds)))
	    (if (not (gnusputi-add-entry-or-insert-field "Reply-To" change-reply-to))
		(progn
		  (ding)
		  (message (concat "Cannot add an address '" change-reply-to "' into Reply-To field."))
		  (if gnusputi-iterate-change-reply-to (sleep-for gnusputi-sleep-seconds)))))))
      (if (not gnusputi-iterate-change-reply-to) (setq continue nil))
      (if changed (message "Mark set")))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Subject field operation
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun gnusputi-change-subject ()
  "Move point to end of Subject field, and change subject entry."
  (interactive)
  (expand-abbrev)
  (let ((cur-subject (gnusputi-current-entry "Subject"))
	new-subject)
    (setq new-subject (read-string "Change Subject: "
				   (if cur-subject
				       (format gnusputi-prev-subject-format cur-subject)
				     nil)))
    (push-mark (point) t) ;not display the message "Mark set"
    (if (gnusputi-change-field "Subject" new-subject)
	(message "Mark set")
      (ding)
      (message "Subject field is not changed."))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;; end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(provide 'gnusputi)
